package com.persagy.apm.report.detail.paragraphvalue.controller;

import com.persagy.apm.report.detail.paragraphvalue.service.IReportParagraphValueService;

import org.springframework.beans.factory.annotation.Autowired;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import com.persagy.apm.report.detail.paragraphvalue.model.*;
import com.persagy.apm.report.detail.paragraphvalue.model.vo.*;
import com.persagy.apm.report.detail.paragraphvalue.model.dto.*;
import com.persagy.apm.common.response.*;
import com.persagy.apm.common.utils.ResultHelper;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.util.List;

/**
 * 报告段落取值控制层
 *
 * @author lixing
 * @version V1.0 2021-07-14 16:59:56
 */
@ApiIgnore
@Api(tags = "报告段落取值", hidden = true)
@Validated
@RestController
@RequestMapping("reportParagraphValues")
public class ReportParagraphValueController {
    @Autowired
    private IReportParagraphValueService reportParagraphValueService;

    @ApiOperation(value = "创建报告段落取值")
    @PostMapping("/create")
    public CommonResult<ApmResponseUpsertVO> createReportParagraphValue(
            @Valid @RequestBody AddReportParagraphValueDTO addReportParagraphValueDTO) {
        String id = reportParagraphValueService.createReportParagraphValue(addReportParagraphValueDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(id));
    }

    @ApiOperation(value = "报告段落取值详情")
    @PostMapping("/get")
    public CommonResult<ReportParagraphValueItemVO> queryReportParagraphValueDetail(
            @Valid @RequestBody QueryReportParagraphValueDetailDTO queryReportParagraphValueDetailDTO) {
        ReportParagraphValue reportParagraphValue = reportParagraphValueService.queryReportParagraphValueDetail(queryReportParagraphValueDetailDTO.getId());
        ReportParagraphValueItemVO reportParagraphValueItemVO = ConvertReportParagraphValueTool.INSTANCE.convert2ResponseItemDTO(reportParagraphValue);
        return ResultHelper.single(reportParagraphValueItemVO);
    }


    @ApiOperation(value = "更新报告段落取值")
    @PostMapping("/update")
    public CommonResult<ApmResponseUpsertVO> updateReportParagraphValue(
            @Valid @RequestBody UpdateReportParagraphValueDTO updateReportParagraphValueDTO) {
        reportParagraphValueService.updateReportParagraphValue(updateReportParagraphValueDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(updateReportParagraphValueDTO.getId()));
    }


    @ApiOperation(value = "删除报告段落取值")
    @PostMapping("/delete")
    public CommonResult deleteReportParagraphValue(
            @RequestBody @Valid DeleteReportParagraphValueDTO deleteReportParagraphValueDTO) {
        reportParagraphValueService.deleteReportParagraphValue(deleteReportParagraphValueDTO.getId());
        return ResultHelper.success();
    }


    @ApiOperation(value = "报告段落取值列表")
    @PostMapping("/query")
    public CommonResult<PageList<ReportParagraphValueListItemVO>> queryReportParagraphValue(
            @Valid @RequestBody QueryReportParagraphValueDTO queryReportParagraphValueDTO) {
        List<ReportParagraphValue> reportParagraphValueList = reportParagraphValueService.queryReportParagraphValueList(queryReportParagraphValueDTO);
        List<ReportParagraphValueListItemVO> resultList = ConvertReportParagraphValueTool.INSTANCE.convert2List(reportParagraphValueList);
        return ResultHelper.multi(resultList);
    }

    //@ApiOperation(value = "报告段落取值列表, 分页查询")
    //@PostMapping("/query")
    //public CommonResult<PageList<ResponseReportParagraphValueListItemVO>> queryReportParagraphValue(
    //        @Valid @RequestBody PageQueryReportParagraphValueDTO pageQueryReportParagraphValueDTO) {
    //    IPage<ReportParagraphValue> pagedList = reportParagraphValueService.pageQueryReportParagraphValue(pageQueryReportParagraphValueDTO);
    //    IPage<ResponseReportParagraphValueListItemVO> pagedResultList = pagedList.convert(item -> {
    //        try {
    //           return ConvertReportParagraphValueTool.INSTANCE.convert2ResponseListItemDTO(item);
    //        } catch (Exception e) {
    //            throw new IllegalArgumentException("分页查询，ReportParagraphValue实体类转换为列表对象发生异常", e);
    //        }
    //    });
    //    return ResultHelper.multi(pagedResultList);
    //}
}

