package com.persagy.apm.report.outline.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.persagy.apm.report.dependencies.saasweb.model.vo.SimpleProjectVO;
import com.persagy.apm.report.detail.model.dto.UpdateReportDetailDTO;
import com.persagy.apm.report.detail.model.vo.ReportDetailItemVO;
import com.persagy.apm.report.outline.model.ReportOutline;
import com.persagy.apm.report.outline.model.dto.*;

import java.util.List;

/**
 * 报告概要(ReportOutline) service接口
 *
 * @author lixing
 * @version V1.0 2021-05-17 11:09:37
 */
public interface IReportOutlineService {

    /**
     * 创建报告概要
     *
     * @param addReportOutlineDTO addDTO
     * @return 报告概要主键
     * @author lixing
     * @version V1.0 2021-05-17 11:09:37
     */
    String createReportOutline(AddReportOutlineDTO addReportOutlineDTO);

    /**
     * 获取报告概要的所属类型
     *
     * @param reportOutline 报告概要
     * @return 报告概要的所属类型
     * @author lixing
     * @version V1.0 2021/6/8 5:26 下午
     */
    String getBelongType(ReportOutline reportOutline);

    /**
     * 获取报告概要的业态列表
     *
     * @param reportOutline 报告概要
     * @return 报告概要的业态列表
     * @author lixing
     * @version V1.0 2021/6/8 5:26 下午
     */
    List<String> getBuildingTypes(ReportOutline reportOutline);

    /**
     * 报告概要详情
     *
     * @param id 主键
     * @return 报告概要do对象
     * @author lixing
     * @version V1.0 2021-05-17 11:09:37
     */
    ReportOutline queryReportOutlineDetail(String id);

    /**
     * 获取商业报告详情信息
     *
     * @param reportOutlineId 主键
     * @return 商业报告详情信息
     * @author lixing
     * @version V1.0 2021/5/31 6:24 下午
     */
    ReportDetailItemVO getReportDetailInfo(String reportOutlineId);

    /**
     * 更新报告概要
     *
     * @param updateReportOutlineDTO updateDTO
     * @author lixing
     * @version V1.0 2021-05-17 11:09:37
     */
    void updateReportOutline(UpdateReportOutlineDTO updateReportOutlineDTO);

    /**
     * 更新报告概要
     *
     * @param reportOutline 报告概要对象
     * @author lixing
     * @version V1.0 2021-05-17 11:09:37
     */
    void updateReportOutline(ReportOutline reportOutline);

    /**
     * 查询和报告类型相关的项目id列表
     *
     * @param reportOutline 报告概要对象
     * @return 项目id列表
     * @author lixing
     * @version V1.0 2021/5/30 8:48 下午
     */
    List<String> queryRelatedProjectIds(ReportOutline reportOutline);

    /**
     * 批量删除报告概要
     *
     * @param ids 主键列表
     * @author lixing
     * @version V1.0 2021-05-17 11:09:37
     */
    void batchDeleteReportOutline(List<String> ids);

    /**
     * 查询报告概要
     *
     * @param queryReportOutlineDTO queryDTO
     * @return List<ReportOutline>
     * @author lixing
     * @version V1.0 2021-05-17 11:09:37
     */
    List<ReportOutline> queryReportOutlineList(QueryReportOutlineDTO queryReportOutlineDTO);

    /**
     * 分页查询报告概要
     *
     * @param pageQueryReportOutlineDTO pageQueryDTO
     * @return IPage<ReportOutline>
     * @author lixing
     * @version V1.0 2021-05-17 11:09:37
     */
    IPage<ReportOutline> pageQueryReportOutline(PageQueryReportOutlineDTO pageQueryReportOutlineDTO);

    /**
     * 查询和报告相关的项目列表
     *
     * @param reportOutline 报告概要对象
     * @return 项目列表
     * @author lixing
     * @version V1.0 2021/5/30 8:48 下午
     */
    List<SimpleProjectVO> queryRelatedProjects(ReportOutline reportOutline);

    /**
     * 获取分区下的项目（当前用户权限和报告类型也是项目列表的过滤条件）
     *
     * @param reportOutline 报告概要对象
     * @param areaId 分区id
     * @return 分区下的项目
     * @author lixing
     * @version V1.0 2021/7/22 2:18 下午
     */
    List<SimpleProjectVO> queryAreaSonProjects(ReportOutline reportOutline, String areaId);

    /**
     * 重新生成报告
     *
     * @param recreateReportOutlineDTO 重新生成报告入参
     * @return 报告id
     * @author lixing
     * @version V1.0 2021/5/31 9:47 上午
     */
    String recreateReportOutline(RecreateReportOutlineDTO recreateReportOutlineDTO);

    /**
     * 获取报告所属名称
     *
     * @param reportBelongType 报告所属类型
     * @param belongId 报告所属id
     * @return 报告所属中文名
     * @author lixing
     * @version V1.0 2021/6/3 4:19 下午
     */
    String getBelongName(String reportBelongType, String belongId);

    /**
     * 更新报告详情
     *
     * @param updateReportDetailDTO 更新参数
     * @author lixing
     * @version V1.0 2021/8/20 3:16 下午
     */
    void updateReportDetail(UpdateReportDetailDTO updateReportDetailDTO);
}
