package com.persagy.apm.report.paragraphs.service.impl.energybudget;

import com.persagy.apm.report.detail.enums.AttrValueTypeEnum;
import com.persagy.apm.report.detail.model.vo.AttrValueVO;
import com.persagy.apm.report.detail.model.vo.GroupVO;
import com.persagy.apm.report.detail.model.vo.ItemVO;
import com.persagy.apm.report.detail.paragraphvalue.model.vo.ParagraphContentVO;
import com.persagy.apm.report.indicator.codes.budget.BudgetHeaderCodes;
import com.persagy.apm.report.outline.model.ReportOutline;
import com.persagy.apm.report.outline.service.IReportOutlineService;
import com.persagy.apm.report.paragraphs.service.impl.AreaDepsParagraphBuilder;
import com.persagy.apm.report.paragraphs.service.impl.ProjectParagraphBuilder;
import org.apache.commons.lang3.StringUtils;
import org.mockito.internal.util.collections.Sets;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * 能耗预算多项目段落建造者
 *
 * @author lixing
 * @version V1.0 2021/7/26 4:52 下午
 **/
@Service
public class AreaEnergyBudgetBuilder extends AreaDepsParagraphBuilder {
    @Autowired
    ProjectEnergyBudgetBuilder projectEnergyBudgetBuilder;
    @Autowired
    IReportOutlineService reportOutlineService;

    @Override
    public List<AttrValueVO> getHeader(
            ParagraphContentVO paragraphContentVO, ReportOutline reportOutline) {
        // 将项目统计信息转换为map
        List<AttrValueVO> summary = paragraphContentVO.getSummary();
        Set<String> codeSet = Sets.newSet(
                BudgetHeaderCodes.monthlyBudget.name(),
                BudgetHeaderCodes.currentMonth.name()
        );

        Map<String, AttrValueVO> data = attrValueList2Map(codeSet, summary);

        /* 拼装返回信息 */
        List<AttrValueVO> result = new ArrayList<>();
        // 总预算
        AttrValueVO monthlyBudget = data.get(BudgetHeaderCodes.monthlyBudget.name());
        if (monthlyBudget != null) {
            result.add(new AttrValueVO(
                    BudgetHeaderCodes.monthlyBudget.name(), monthlyBudget.getValue(),
                    monthlyBudget.getType(), monthlyBudget.getUnit()));
        }

        // 实际能耗
        AttrValueVO currentMonth = data.get(BudgetHeaderCodes.currentMonth.name());
        if (currentMonth != null) {
            result.add(new AttrValueVO(
                    BudgetHeaderCodes.currentMonth.name(), currentMonth.getValue(),
                    currentMonth.getType(), currentMonth.getUnit()
            ));
        }

        // 获取能耗超过预算的子分区数目，如果没有子分区，按项目数统计
        Integer excessBudgetDepCountValue = getExcessBudgetDepCount(paragraphContentVO);
        String excessBudgetDepCountValueStr = excessBudgetDepCountValue == null ?
                null : String.valueOf(excessBudgetDepCountValue);
        AttrValueVO excessBudgetDepCount = new AttrValueVO(
                BudgetHeaderCodes.excessBudgetDepCount.name(), excessBudgetDepCountValueStr,
                AttrValueTypeEnum.number);
        result.add(excessBudgetDepCount);

        return result;
    }

    /**
     * 获取超过预算的组织数量
     *
     * @param paragraphContentVO 预算段落内容对象，包含每个组织的预算情况
     * @return 超过预算的组织数量
     * @author lixing
     * @version V1.0 2021/8/2 2:01 下午
     */
    private Integer getExcessBudgetDepCount(ParagraphContentVO paragraphContentVO) {
        Integer excessBudgetDepCountValue = null;
        List<ItemVO> projectBudgetList = paragraphContentVO.getItems();
        if (projectBudgetList != null) {
            for (ItemVO itemVO : projectBudgetList) {
                List<AttrValueVO> attrs = itemVO.getAttrs();
                Set<String> tmpCodeSet = Sets.newSet(
                        BudgetHeaderCodes.monthlyProportion.name()
                );

                Map<String, AttrValueVO> projectData = attrValueList2Map(tmpCodeSet, attrs);
                String monthlyProportion = projectData.get(BudgetHeaderCodes.monthlyProportion.name()).getValue();
                if (StringUtils.isBlank(monthlyProportion)) {
                    continue;
                }
                double monthlyProportionDoubleValue = Double.parseDouble(monthlyProportion);
                if (monthlyProportionDoubleValue > 1d) {
                    if (excessBudgetDepCountValue == null) {
                        excessBudgetDepCountValue = 1;
                    } else {
                        excessBudgetDepCountValue += 1;
                    }
                }
            }
        }
        return excessBudgetDepCountValue;
    }

    @Override
    public List<AttrValueVO> getSummary(List<GroupVO> groupVOList, List<ItemVO> itemVOList, List<String> projectIdList) {
        return projectEnergyBudgetBuilder.getSummary(groupVOList, itemVOList, projectIdList);
    }

    @Override
    public ProjectParagraphBuilder getProjectParagraphBuilder() {
        return projectEnergyBudgetBuilder;
    }
}
