package com.persagy.apm.report.paragraphs.service.impl.energyefficiency;

import com.google.gson.Gson;
import com.persagy.apm.report.common.utils.DataUtils;
import com.persagy.apm.report.dependencies.saasweb.model.vo.SimpleProjectVO;
import com.persagy.apm.report.dependencies.saasweb.service.ISaasWebService;
import com.persagy.apm.report.detail.enums.AttrValueTypeEnum;
import com.persagy.apm.report.detail.model.vo.AttrValueVO;
import com.persagy.apm.report.detail.model.vo.GroupVO;
import com.persagy.apm.report.detail.model.vo.ItemVO;
import com.persagy.apm.report.detail.paragraphvalue.model.vo.ParagraphContentVO;
import com.persagy.apm.report.indicator.codes.energyefficiency.AreaEnergyEfficiencyContentCodes;
import com.persagy.apm.report.indicator.codes.energyefficiency.ProjectEnergyEfficiencyContentCodes;
import com.persagy.apm.report.indicator.factory.EnergyEfficiencyFactory;
import com.persagy.apm.report.indicator.model.vo.GraphItemVO;
import com.persagy.apm.report.indicator.model.vo.GraphVO;
import com.persagy.apm.report.indicator.utils.IndicatorUtils;
import com.persagy.apm.report.outline.model.ReportOutline;
import com.persagy.apm.report.paragraphs.constants.StandardNames;
import com.persagy.apm.report.paragraphs.service.impl.AreaIndicatorsParagraphBuilder;
import com.persagy.apm.report.userconfig.function.service.IFunctionService;
import com.persagy.apm.report.userconfig.paragraph.service.IParagraphService;
import com.persagy.apm.report.userconfig.pargraphdefine.service.IParagraphDefineService;
import com.persagy.apm.report.userconfig.statisticsitems.enums.QualifiedEnum;
import org.mockito.internal.util.collections.Sets;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 能效情况多项目段落建造者
 *
 * @author lixing
 * @version V1.0 2021/7/26 4:52 下午
 **/
@Service
public class AreaEnergyEfficiencyBuilder extends AreaIndicatorsParagraphBuilder {
    @Autowired
    EnergyEfficiencyFactory energyEfficiencyFactory;
    @Autowired
    IParagraphDefineService paragraphDefineService;
    @Autowired
    IParagraphService paragraphService;
    @Autowired
    IFunctionService functionService;
    @Autowired
    ProjectEnergyEfficiencyBuilder projectEnergyEfficiencyBuilder;
    @Autowired
    ISaasWebService saasWebService;

    @Override
    public List<AttrValueVO> getHeader(
            ParagraphContentVO paragraphContentVO, ReportOutline reportOutline) {
        return null;
    }

    @Override
    public List<AttrValueVO> getSummary(
            List<GroupVO> groupVOList, List<ItemVO> itemVOList, List<String> projectIdList) {
        return null;
    }

    @Override
    public ItemVO getIndicatorItem(Date reportDate, List<String> projectIds, String itemId) {
        Set<String> sumCodeSet = Sets.newSet(
                ProjectEnergyEfficiencyContentCodes.currentMonth.name(),
                ProjectEnergyEfficiencyContentCodes.lastMonth.name(),
                ProjectEnergyEfficiencyContentCodes.chainGrowth.name()
        );

        Set<String> statisticCodeSet = Sets.newSet(
                ProjectEnergyEfficiencyContentCodes.isQualified.name(),
                ProjectEnergyEfficiencyContentCodes.currentMonth.name()
        );

        List<ItemVO> projectItems = new ArrayList<>();
        // 不合格项目数
        Integer unqualifiedProjectCount = null;
        // 各项目能效值
        List<GraphItemVO> projectsEfficiency = new ArrayList<>();
        for (String projectId : projectIds) {
            SimpleProjectVO simpleProjectInfo = saasWebService.getSimpleProjectInfo(projectId);
            if (simpleProjectInfo == null) {
                continue;
            }
            ItemVO projectItemValue = projectEnergyEfficiencyBuilder.getProjectItemValue(
                    itemId, projectId, reportDate);
            Map<String, AttrValueVO> attrValueMap = attrValueList2Map(statisticCodeSet, projectItemValue.getAttrs());

            // 统计各项目能效值
            GraphItemVO graphItemVO = new GraphItemVO(simpleProjectInfo.getProjectLocalName(),
                    DataUtils.parseDouble(attrValueMap.get(ProjectEnergyEfficiencyContentCodes.currentMonth.name()).getValue()),
                    AttrValueTypeEnum.number_2, "");
            projectsEfficiency.add(graphItemVO);
            // 统计不合格项目数
            String isQualifiedDesc = attrValueMap.get(ProjectEnergyEfficiencyContentCodes.isQualified.name()).getValue();
            if (QualifiedEnum.NOT_QUALIFIED.getDesc().equals(isQualifiedDesc)) {
                unqualifiedProjectCount = unqualifiedProjectCount == null ? 1 : unqualifiedProjectCount + 1;
            }

            projectItems.add(projectItemValue);
        }
        Map<String, AttrValueVO> sum = getSum(null, projectItems, sumCodeSet);

        ItemVO itemVO = new ItemVO();
        List<AttrValueVO> attrs = initItem(itemVO, itemId);

        // 本月能效
        AttrValueVO currentMonth = new AttrValueVO(AreaEnergyEfficiencyContentCodes.currentMonth.name(),
                IndicatorUtils.getAttrStrValue(sum.get(ProjectEnergyEfficiencyContentCodes.currentMonth.name())),
                AttrValueTypeEnum.number_2);
        attrs.add(currentMonth);
        // 上月能效
        AttrValueVO lastMonth = new AttrValueVO(AreaEnergyEfficiencyContentCodes.lastMonth.name(),
                IndicatorUtils.getAttrStrValue(sum.get(ProjectEnergyEfficiencyContentCodes.lastMonth.name())),
                AttrValueTypeEnum.number_2);
        attrs.add(lastMonth);
        // 环比增量
        AttrValueVO chainGrowth = new AttrValueVO(AreaEnergyEfficiencyContentCodes.chainGrowth.name(),
                IndicatorUtils.getAttrStrValue(sum.get(ProjectEnergyEfficiencyContentCodes.chainGrowth.name())),
                AttrValueTypeEnum.number_2);
        attrs.add(chainGrowth);
        // 环比增幅
        AttrValueVO chainRange = energyEfficiencyFactory.getChainRange(
                DataUtils.parseDouble(chainGrowth.getValue()), DataUtils.parseDouble(lastMonth.getValue()));
        attrs.add(chainRange);
        // 合格标准
        AttrValueVO qualifiedStandard = energyEfficiencyFactory.getQualifiedStandard(
                functionService.queryFunctionDetail(itemId));
        attrs.add(qualifiedStandard);
        // 总项目数
        Integer projectsCountValue = projectIds.size();
        AttrValueVO projectsCount = new AttrValueVO(
                AreaEnergyEfficiencyContentCodes.projectsCount.name(),
                String.valueOf(projectsCountValue),
                AttrValueTypeEnum.number);
        attrs.add(projectsCount);
        // 不合格项目数
        AttrValueVO unqualifiedCount = new AttrValueVO(
                AreaEnergyEfficiencyContentCodes.unqualifiedCount.name(),
                unqualifiedProjectCount == null ? null : String.valueOf(unqualifiedProjectCount),
                AttrValueTypeEnum.number);
        attrs.add(unqualifiedCount);
        // 不合格率
        AttrValueVO unqualifiedProportion = energyEfficiencyFactory.getUnqualifiedProportion(
                IndicatorUtils.getAttrDoubleValue(projectsCount), IndicatorUtils.getAttrDoubleValue(unqualifiedCount));
        attrs.add(unqualifiedProportion);
        /* 各项目能效值 */
        // 按从高到低排序
        Map<String, Integer> sortMap = new HashMap<>();
        sortMap.put("value", -1);
        DataUtils.sort(projectsEfficiency, sortMap);

        Gson gson = new Gson();
        GraphVO graphVO = new GraphVO(StandardNames.STANDARD_VALUE, qualifiedStandard.getValue(),
                AttrValueTypeEnum.number_2, projectsEfficiency);
        AttrValueVO projectEfficiencies = new AttrValueVO(
                AreaEnergyEfficiencyContentCodes.projectEfficiencies.name(),
                gson.toJson(graphVO), AttrValueTypeEnum.graph);
        attrs.add(projectEfficiencies);
        return itemVO;
    }
}