package com.persagy.apm.report.paragraphs.service.impl.env;

import com.alibaba.excel.util.CollectionUtils;
import com.google.gson.Gson;
import com.google.gson.JsonSyntaxException;
import com.persagy.apm.report.common.utils.DataUtils;
import com.persagy.apm.report.userconfig.function.model.Function;
import com.persagy.apm.report.userconfig.function.service.IFunctionService;
import com.persagy.apm.report.detail.enums.AttrValueTypeEnum;
import com.persagy.apm.report.detail.model.vo.AttrValueVO;
import com.persagy.apm.report.detail.model.vo.GroupVO;
import com.persagy.apm.report.detail.model.vo.ItemVO;
import com.persagy.apm.report.detail.paragraphvalue.model.vo.ParagraphContentVO;
import com.persagy.apm.report.indicator.codes.env.ProjectEnvContentCodes;
import com.persagy.apm.report.indicator.codes.env.ProjectEnvHeaderCodes;
import com.persagy.apm.report.indicator.factory.EnvFactory;
import com.persagy.apm.report.indicator.model.entity.EnvDefine;
import com.persagy.apm.report.outline.model.ReportOutline;
import com.persagy.apm.report.paragraphs.constants.ItemAttrConstants;
import com.persagy.apm.report.paragraphs.service.impl.ProjectParagraphBuilder;
import com.persagy.apm.report.paragraphs.utils.ParagraphUtils;
import org.mockito.internal.util.collections.Sets;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 环境单项目段落建造者
 *
 * @author lixing
 * @version V1.0 2021/7/26 4:52 下午
 **/
@Service
public class ProjectEnvBuilder extends ProjectParagraphBuilder {
    @Autowired
    EnvFactory envFactory;
    @Autowired
    IFunctionService functionService;

    @Override
    public List<AttrValueVO> getHeader(
            ParagraphContentVO paragraphContentVO, ReportOutline reportOutline) {
        List<AttrValueVO> results = new ArrayList<>();

        List<AttrValueVO> summary = paragraphContentVO.getSummary();
        Set<String> codeSet = Sets.newSet(
                ProjectEnvContentCodes.alarmCount.name()
        );
        Map<String, AttrValueVO> dataMap = attrValueList2Map(codeSet, summary);

        // 总报警数
        AttrValueVO alarmCount = dataMap.get(ProjectEnvHeaderCodes.alarmCount.name());
        if (alarmCount != null) {
            results.add(new AttrValueVO(
                    ProjectEnvHeaderCodes.alarmCount.name(), alarmCount.getValue(), alarmCount.getType()));
        }

        // 报警次数最多的指标
        String maxAlarmIndicator = null;
        // 处理率最低的指标
        String minHandleProportionIndicator = null;

        List<ItemVO> items = paragraphContentVO.getItems();
        Set<String> attrCodeSet = Sets.newSet(
                ProjectEnvContentCodes.alarmCount.name(),
                ProjectEnvContentCodes.handleProportion.name()
        );
        if (!CollectionUtils.isEmpty(items)) {
            // 将段落中包含的条目转换为map，进行排序
            List<Map<String, String>> itemMapList = items.stream().map(
                    item -> ParagraphUtils.item2Map(item, attrCodeSet)).collect(Collectors.toList());

            // 报警次数最多的指标
            List<Map<String, String>> alarmCountMap = itemMapList.stream().filter(
                    map -> map.get(ProjectEnvContentCodes.alarmCount.name()) != null).
                    collect(Collectors.toList());
            if (!CollectionUtils.isEmpty(alarmCountMap)) {
                Map<String, Integer> alarmCountSortMap = new HashMap<>();
                alarmCountSortMap.put(ProjectEnvContentCodes.alarmCount.name(), -1);
                DataUtils.sort(alarmCountMap, alarmCountSortMap);
                maxAlarmIndicator = alarmCountMap.get(0).get(ItemAttrConstants.NAME);
            }


            // 处理率最低的指标
            List<Map<String, String>> handleProportionMap = itemMapList.stream().filter(
                    map -> map.get(ProjectEnvContentCodes.handleProportion.name()) != null).
                    collect(Collectors.toList());
            if (!CollectionUtils.isEmpty(handleProportionMap)) {
                Map<String, Integer> handleProportionSortMap = new HashMap<>();
                handleProportionSortMap.put(ProjectEnvContentCodes.handleProportion.name(), 1);
                DataUtils.sort(handleProportionMap, handleProportionSortMap);
                minHandleProportionIndicator = handleProportionMap.get(0).get(ItemAttrConstants.NAME);
            }
        }

        AttrValueVO maxAlarmIndicatorVO = new AttrValueVO(
                ProjectEnvHeaderCodes.maxAlarmIndicator.name(), maxAlarmIndicator, AttrValueTypeEnum.str);
        results.add(maxAlarmIndicatorVO);

        AttrValueVO minHandleProportionIndicatorVO = new AttrValueVO(
                ProjectEnvHeaderCodes.minHandleProportionIndicator.name(), minHandleProportionIndicator, AttrValueTypeEnum.str);
        results.add(minHandleProportionIndicatorVO);

        return results;
    }

    @Override
    public List<AttrValueVO> getSummary(
            List<GroupVO> groupVOList, List<ItemVO> itemVOList, List<String> projectIdList) {
        Set<String> codeSet = Sets.newSet(
                ProjectEnvContentCodes.alarmCount.name()
        );
        Map<String, AttrValueVO> sum = getSum(groupVOList, itemVOList, codeSet);
        return map2ValueList(sum);
    }

    @Override
    public ItemVO getProjectItemValue(String itemId, String projectId, Date reportMonth) {
        // 获取指标的定义，转换为指标对象
        Function function = functionService.queryFunctionDetail(itemId);
        Gson gson = new Gson();
        EnvDefine envDefine;
        try {
            envDefine = gson.fromJson(function.getDefine(), EnvDefine.class);
            if (envDefine.getType() == null || envDefine.getParentType() == null) {
                throw new IllegalArgumentException("环境指标定义配置错误，错误的定义内容为：" + function);
            }
        } catch (JsonSyntaxException e) {
            throw new IllegalArgumentException("环境指标定义配置错误，错误的定义内容为：" + function);
        }
        // 从指标库中获取指标取值
        return envFactory.getEnvInfo(function, envDefine, projectId, reportMonth);
    }

    @Override
    public List<AttrValueVO> getProjectSummary(ParagraphContentVO projectParagraphContent) {
        return null;
    }
}
