package com.persagy.apm.report.paragraphs.service.impl.iot;

import com.alibaba.excel.util.CollectionUtils;
import com.google.gson.Gson;
import com.google.gson.JsonSyntaxException;
import com.persagy.apm.report.common.constants.DataConstants;
import com.persagy.apm.report.common.utils.DataUtils;
import com.persagy.apm.report.detail.enums.AttrValueTypeEnum;
import com.persagy.apm.report.detail.model.vo.AttrValueVO;
import com.persagy.apm.report.detail.model.vo.GroupVO;
import com.persagy.apm.report.detail.model.vo.ItemVO;
import com.persagy.apm.report.detail.paragraphvalue.model.vo.ParagraphContentVO;
import com.persagy.apm.report.indicator.codes.iot.ProjectIotContentCodes;
import com.persagy.apm.report.indicator.codes.iot.ProjectIotHeaderCodes;
import com.persagy.apm.report.indicator.factory.IotFactory;
import com.persagy.apm.report.indicator.model.entity.IotDefine;
import com.persagy.apm.report.indicator.utils.IndicatorUtils;
import com.persagy.apm.report.outline.model.ReportOutline;
import com.persagy.apm.report.paragraphs.service.impl.ProjectParagraphBuilder;
import com.persagy.apm.report.userconfig.function.model.Function;
import com.persagy.apm.report.userconfig.function.service.IFunctionService;
import org.mockito.internal.util.collections.Sets;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 单项目数据质量段落建造者
 *
 * @author lixing
 * @version V1.0 2021/8/6 9:53 上午
 **/
@Service
public class ProjectIotBuilder extends ProjectParagraphBuilder {
    @Autowired
    IFunctionService functionService;
    @Autowired
    IotFactory iotFactory;

    @Override
    public List<AttrValueVO> getHeader(ParagraphContentVO paragraphContentVO, ReportOutline reportOutline) {
        List<AttrValueVO> results = new ArrayList<>();

        List<ItemVO> items = paragraphContentVO.getItems();
        String itemsSummary = null;
        Set<String> codeSet = Sets.newSet(
                ProjectIotContentCodes.currentMonth.name(),
                ProjectIotContentCodes.isQualified.name()
        );
        if (!CollectionUtils.isEmpty(items)) {
            for (ItemVO item : items) {
                String name = item.getName();
                List<AttrValueVO> attrs = item.getAttrs();
                Map<String, AttrValueVO> attrMap = attrValueList2Map(codeSet, attrs);
                Double attrDoubleValue = IndicatorUtils.getAttrDoubleValue(
                        attrMap.get(ProjectIotContentCodes.currentMonth.name()));
                if (attrDoubleValue != null) {
                    attrDoubleValue = DataUtils.doubleMultiply(attrDoubleValue, 100d);
                }
                String attrValueStr = attrDoubleValue == null ? "--" : String.valueOf(attrDoubleValue);
                String isQualifiedStr = attrMap.get(ProjectIotContentCodes.isQualified.name()).getValue();
                isQualifiedStr = isQualifiedStr == null ? "" : isQualifiedStr;
                String itemInfo = name + ":" +
                        attrValueStr + DataConstants.PERCENT + " " + isQualifiedStr;

                itemsSummary = itemsSummary == null ? itemInfo : itemsSummary + "," + itemInfo;
            }
        }
        AttrValueVO itemsSummaryVO = new AttrValueVO(
                ProjectIotHeaderCodes.currentMonth.name(), itemsSummary, AttrValueTypeEnum.str);
        results.add(itemsSummaryVO);

        return results;
    }

    @Override
    public List<AttrValueVO> getSummary(List<GroupVO> groupVOList, List<ItemVO> itemVOList, List<String> projectIdList) {
        return null;
    }

    @Override
    public ItemVO getProjectItemValue(String itemId, String projectId, Date reportMonth) {
        // 获取指标的定义，转换为指标对象
        Function function = functionService.queryFunctionDetail(itemId);
        Gson gson = new Gson();
        IotDefine iotDefine;
        try {
            iotDefine = gson.fromJson(function.getDefine(), IotDefine.class);
            if (iotDefine.getTargetCode() == null) {
                throw new IllegalArgumentException("数据质量指标定义配置错误，错误的定义内容为：" + function);
            }
        } catch (JsonSyntaxException e) {
            throw new IllegalArgumentException("数据质量指标定义配置错误，错误的定义内容为：" + function);
        }
        // 从指标库中获取指标取值
        return iotFactory.getIotInfo(function, iotDefine, projectId, reportMonth);
    }

    @Override
    public List<AttrValueVO> getProjectSummary(ParagraphContentVO projectParagraphContent) {
        return null;
    }
}
