package com.persagy.apm.report.userconfig.paragraphmodel.model;

import org.mapstruct.*;
import org.mapstruct.MappingTarget;
import org.mapstruct.factory.Mappers;
import com.persagy.apm.report.userconfig.paragraphmodel.model.vo.*;
import com.persagy.apm.report.userconfig.paragraphmodel.model.dto.*;

import java.util.List;

/**
 * 段落模板(ParagraphModel) dto、vo、do转换工具类
 *
 * @author lixing
 * @version V1.0 2021-07-20 10:26:14 2021-07-20 10:26:14
 */
@Mapper(nullValueMappingStrategy = NullValueMappingStrategy.RETURN_DEFAULT,
        nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS)
public interface ConvertParagraphModelTool {

    ConvertParagraphModelTool INSTANCE = Mappers.getMapper(ConvertParagraphModelTool.class);

    /**
     * do转换为ResponseItemVO
     *
     * @param paragraphModel do对象
     * @return ResponseItemVO
     * @author lixing
     * @version V1.0 2021-07-20 10:26:14
     */
    ParagraphModelItemVO convert2ResponseItemDTO(ParagraphModel paragraphModel);

    /**
     * do转换为ResponseListItemVO
     *
     * @param paragraphModel do对象
     * @return ResponseListItemVO
     * @author lixing
     * @version V1.0 2021-07-20 10:26:14
     */
    ParagraphModelListItemVO convert2ResponseListItemDTO(ParagraphModel paragraphModel);

    /**
     * do列表转换为ResponseListItemVO列表
     *
     * @param paragraphModelList do列表
     * @return ResponseListItemVO列表
     * @author lixing
     * @version V1.0 2021-07-20 10:26:14
     */
    List<ParagraphModelListItemVO> convert2List(List<ParagraphModel> paragraphModelList);

    /**
     * addDTO转换为do
     *
     * @param addParagraphModelDTO addDTO
     * @return 要创建的do对象
     * @author lixing
     * @version V1.0 2021-07-20 10:26:14
     */
    ParagraphModel convertAddDto2Entity(AddParagraphModelDTO addParagraphModelDTO);

    /**
     * updateDTO转换为实体
     *
     * @param paragraphModel          更新前的do对象
     * @param updateParagraphModelDTO updateDTO
     * @return 更新后的do对象
     * @author lixing
     * @version V1.0 2021-07-20 10:26:14
     */
    ParagraphModel convertUpdateDto2Entity(@MappingTarget ParagraphModel paragraphModel, UpdateParagraphModelDTO updateParagraphModelDTO);
}

