package com.persagy.apm.report.userconfig.pargraphdefine.controller;

import com.persagy.apm.report.userconfig.pargraphdefine.service.IParagraphDefineService;

import org.springframework.beans.factory.annotation.Autowired;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import com.persagy.apm.report.userconfig.pargraphdefine.model.*;
import com.persagy.apm.report.userconfig.pargraphdefine.model.vo.*;
import com.persagy.apm.report.userconfig.pargraphdefine.model.dto.*;
import com.persagy.apm.common.response.*;
import com.persagy.apm.common.utils.ResultHelper;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.util.List;

/**
 * 段落定义控制层
 *
 * @author lixing
 * @version V1.0 2021-07-20 10:23:07
 */
@Api(tags = "段落定义")
@ApiIgnore
@Validated
@RestController
@RequestMapping("paragraphDefines")
public class ParagraphDefineController {
    @Autowired
    private IParagraphDefineService paragraphDefineService;

    @ApiOperation(value = "创建段落定义")
    @PostMapping("/create")
    public CommonResult<ApmResponseUpsertVO> createParagraphDefine(
            @Valid @RequestBody AddParagraphDefineDTO addParagraphDefineDTO) {
        String id = paragraphDefineService.createParagraphDefine(addParagraphDefineDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(id));
    }

    @ApiOperation(value = "段落定义详情")
    @PostMapping("/get")
    public CommonResult<ParagraphDefineItemVO> queryParagraphDefineDetail(
            @Valid @RequestBody QueryParagraphDefineDetailDTO queryParagraphDefineDetailDTO) {
        ParagraphDefine paragraphDefine = paragraphDefineService.queryParagraphDefineDetail(queryParagraphDefineDetailDTO.getId());
        ParagraphDefineItemVO paragraphDefineItemVO = ConvertParagraphDefineTool.INSTANCE.convert2ResponseItemDTO(paragraphDefine);
        return ResultHelper.single(paragraphDefineItemVO);
    }


    @ApiOperation(value = "更新段落定义")
    @PostMapping("/update")
    public CommonResult<ApmResponseUpsertVO> updateParagraphDefine(
            @Valid @RequestBody UpdateParagraphDefineDTO updateParagraphDefineDTO) {
        paragraphDefineService.updateParagraphDefine(updateParagraphDefineDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(updateParagraphDefineDTO.getId()));
    }


    @ApiOperation(value = "删除段落定义")
    @PostMapping("/delete")
    public CommonResult deleteParagraphDefine(
            @RequestBody @Valid DeleteParagraphDefineDTO deleteParagraphDefineDTO) {
        paragraphDefineService.deleteParagraphDefine(deleteParagraphDefineDTO.getId());
        return ResultHelper.success();
    }


    @ApiOperation(value = "段落定义列表")
    @PostMapping("/query")
    public CommonResult<PageList<ParagraphDefineListItemVO>> queryParagraphDefine(
            @Valid @RequestBody QueryParagraphDefineDTO queryParagraphDefineDTO) {
        List<ParagraphDefine> paragraphDefineList = paragraphDefineService.queryParagraphDefineList(queryParagraphDefineDTO);
        List<ParagraphDefineListItemVO> resultList = ConvertParagraphDefineTool.INSTANCE.convert2List(paragraphDefineList);
        return ResultHelper.multi(resultList);
    }

    //@ApiOperation(value = "段落定义列表, 分页查询")
    //@PostMapping("/query")
    //public CommonResult<PageList<ParagraphDefineListItemVO>> queryParagraphDefine(
    //        @Valid @RequestBody PageQueryParagraphDefineDTO pageQueryParagraphDefineDTO) {
    //    IPage<ParagraphDefine> pagedList = paragraphDefineService.pageQueryParagraphDefine(pageQueryParagraphDefineDTO);
    //    IPage<ParagraphDefineListItemVO> pagedResultList = pagedList.convert(item -> {
    //        try {
    //           return ConvertParagraphDefineTool.INSTANCE.convert2ResponseListItemDTO(item);
    //        } catch (Exception e) {
    //            throw new IllegalArgumentException("分页查询，ParagraphDefine实体类转换为列表对象发生异常", e);
    //        }
    //    });
    //    return ResultHelper.multi(pagedResultList);
    //}
}

