package com.persagy.apm.report.userconfig.reporttype.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.persagy.apm.report.userconfig.paragraph.model.vo.ParagraphItemVO;
import com.persagy.apm.report.userconfig.reporttype.model.ReportType;
import com.persagy.apm.report.userconfig.reporttype.model.dto.AddReportTypeDTO;
import com.persagy.apm.report.userconfig.reporttype.model.dto.PageQueryReportTypeDTO;
import com.persagy.apm.report.userconfig.reporttype.model.dto.QueryReportTypeDTO;
import com.persagy.apm.report.userconfig.reporttype.model.dto.UpdateReportTypeDTO;
import com.persagy.apm.report.userconfig.reporttype.model.vo.ResponseReportTypeListItemVO;
import com.persagy.apm.report.dependencies.saasweb.model.vo.PartitionProjectVO;
import com.persagy.apm.report.dependencies.saasweb.model.vo.PartitionVO;
import com.persagy.apm.report.dependencies.saasweb.model.vo.SimpleProjectVO;

import java.util.List;

/**
 * 报告类型(ReportType) service接口
 *
 * @author lixing
 * @version V1.0 2021-05-19 10:39:15
 */
public interface IReportTypeService {

    /**
     * 创建报告类型
     *
     * @param addReportTypeDTO addDTO
     * @return 报告类型主键
     * @author lixing
     * @version V1.0 2021-05-19 10:39:15
     */
    String createReportType(AddReportTypeDTO addReportTypeDTO);

    /**
     * 报告类型详情
     *
     * @param id 主键
     * @return 报告类型do对象
     * @author lixing
     * @version V1.0 2021-05-19 10:39:15
     */
    ReportType queryReportTypeDetail(String id);

    /**
     * 更新报告类型
     *
     * @param updateReportTypeDTO updateDTO
     * @author lixing
     * @version V1.0 2021-05-19 10:39:15
     */
    void updateReportType(UpdateReportTypeDTO updateReportTypeDTO);

    /**
     * 删除报告类型
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-05-19 10:39:15
     */
    void deleteReportType(String id);

    /**
     * 查询报告类型
     *
     * @param queryReportTypeDTO queryDTO
     * @return List<ReportType>
     * @author lixing
     * @version V1.0 2021-05-19 10:39:15
     */
    List<ReportType> queryReportTypeList(QueryReportTypeDTO queryReportTypeDTO);

    /**
     * 分页查询报告类型
     *
     * @param pageQueryReportTypeDTO pageQueryDTO
     * @return IPage<ReportType>
     * @author lixing
     * @version V1.0 2021-05-19 10:39:15
     */
    IPage<ReportType> pageQueryReportType(PageQueryReportTypeDTO pageQueryReportTypeDTO);

    /**
     * 查询当前用户可用的报告类型
     * @param belongType 报告所属类型
     * @return 当前用户可用的报告类型
     * @author lixing
     * @version V1.0 2021/5/19 4:39 下午
     */
    IPage<ResponseReportTypeListItemVO> queryAvailableReportTypes(String belongType);

    /**
     * 获取报告类型关联的业态
     *
     * @param reportTypeId 报告类型id
     * @return 报告类型关联的业态列表
     * @author lixing
     * @version V1.0 2021/7/16 11:58 上午
     */
    List<String> getReportTypeBuildingTypes(String reportTypeId);

    /**
     * 根据报告类型获取用户可用的分区项目树
     *
     * @param reportTypeId 报告类型id
     * @return 用户可用的分区项目树
     * @author lixing
     * @version V1.0 2021/7/16 2:34 下午
     */
    List<PartitionProjectVO> getAvailableProjectTree(String reportTypeId);

    /**
     * 根据报告类型获取用户可用的项目列表
     *
     * @param reportTypeId 报告类型id
     * @return 用户可用的项目列表
     * @author lixing
     * @version V1.0 2021/7/16 2:33 下午
     */
    List<SimpleProjectVO> queryAvailableProjectList(String reportTypeId);

    /**
     * 在指定报告类型下，获取当前用户可选的分区的子分区
     *
     * @param reportTypeId 报告类型id
     * @param areaId 分区id
     * @return 子分区列表
     * @author lixing
     * @version V1.0 2021/7/22 10:21 上午
     */
    List<PartitionVO> getSonAreas(String reportTypeId, String areaId);

    /**
     * 获取报告类型关联的段落
     *
     * @param reportTypeId 报告类型主键
     * @return 报告类型关联的段落
     * @author lixing
     * @version V1.0 2021/7/16 7:21 下午
     */
    List<ParagraphItemVO> queryParagraphsByReportType(String reportTypeId);

    /**
     * 根据报告类型获取用户可用的分区列表
     *
     * @param reportTypeId 报告类型id
     * @return 用户可用的分区列表
     * @author lixing
     * @version V1.0 2021/7/16 2:33 下午
     */
    List<PartitionVO> getAvailableAreaTree(String reportTypeId);
}
