package com.persagy.apm.report.userconfig.statisticsitems.controller;

import com.persagy.apm.report.userconfig.statisticsitems.service.IStatisticItemService;

import org.springframework.beans.factory.annotation.Autowired;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import com.persagy.apm.report.userconfig.statisticsitems.model.*;
import com.persagy.apm.report.userconfig.statisticsitems.model.vo.*;
import com.persagy.apm.report.userconfig.statisticsitems.model.dto.*;
import com.persagy.apm.common.response.*;
import com.persagy.apm.common.utils.ResultHelper;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.util.List;

/**
 * 统计条目控制层
 *
 * @author lixing
 * @version V1.0 2021-07-27 16:11:56
 */
@Api(tags = "统计条目")
@ApiIgnore
@Validated
@RestController
@RequestMapping("statisticItems")
public class StatisticItemController {
    @Autowired
    private IStatisticItemService statisticItemService;

    @ApiOperation(value = "创建统计条目")
    @PostMapping("/create")
    public CommonResult<ApmResponseUpsertVO> createStatisticItem(
            @Valid @RequestBody AddStatisticItemDTO addStatisticItemDTO) {
        String id = statisticItemService.createStatisticItem(addStatisticItemDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(id));
    }

    @ApiOperation(value = "统计条目详情")
    @PostMapping("/get")
    public CommonResult<StatisticItemItemVO> queryStatisticItemDetail(
            @Valid @RequestBody QueryStatisticItemDetailDTO queryStatisticItemDetailDTO) {
        StatisticItem statisticItem = statisticItemService.queryStatisticItemDetail(queryStatisticItemDetailDTO.getId());
        StatisticItemItemVO statisticItemItemVO = ConvertStatisticItemTool.INSTANCE.convert2ResponseItemDTO(statisticItem);
        return ResultHelper.single(statisticItemItemVO);
    }


    @ApiOperation(value = "更新统计条目")
    @PostMapping("/update")
    public CommonResult<ApmResponseUpsertVO> updateStatisticItem(
            @Valid @RequestBody UpdateStatisticItemDTO updateStatisticItemDTO) {
        statisticItemService.updateStatisticItem(updateStatisticItemDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(updateStatisticItemDTO.getId()));
    }


    @ApiOperation(value = "删除统计条目")
    @PostMapping("/delete")
    public CommonResult deleteStatisticItem(
            @RequestBody @Valid DeleteStatisticItemDTO deleteStatisticItemDTO) {
        statisticItemService.deleteStatisticItem(deleteStatisticItemDTO.getId());
        return ResultHelper.success();
    }


    @ApiOperation(value = "统计条目列表")
    @PostMapping("/query")
    public CommonResult<PageList<StatisticItemListItemVO>> queryStatisticItem(
            @Valid @RequestBody QueryStatisticItemDTO queryStatisticItemDTO) {
        List<StatisticItem> statisticItemList = statisticItemService.queryStatisticItemList(queryStatisticItemDTO);
        List<StatisticItemListItemVO> resultList = ConvertStatisticItemTool.INSTANCE.convert2List(statisticItemList);
        return ResultHelper.multi(resultList);
    }

    //@ApiOperation(value = "统计条目列表, 分页查询")
    //@PostMapping("/query")
    //public CommonResult<PageList<StatisticItemListItemVO>> queryStatisticItem(
    //        @Valid @RequestBody PageQueryStatisticItemDTO pageQueryStatisticItemDTO) {
    //    IPage<StatisticItem> pagedList = statisticItemService.pageQueryStatisticItem(pageQueryStatisticItemDTO);
    //    IPage<StatisticItemListItemVO> pagedResultList = pagedList.convert(item -> {
    //        try {
    //           return ConvertStatisticItemTool.INSTANCE.convert2ResponseListItemDTO(item);
    //        } catch (Exception e) {
    //            throw new IllegalArgumentException("分页查询，StatisticItem实体类转换为列表对象发生异常", e);
    //        }
    //    });
    //    return ResultHelper.multi(pagedResultList);
    //}
}

