package com.persagy.apm.report.userconfig.statisticsitems.model;

import org.mapstruct.*;
import org.mapstruct.MappingTarget;
import org.mapstruct.factory.Mappers;
import com.persagy.apm.report.userconfig.statisticsitems.model.vo.*;
import com.persagy.apm.report.userconfig.statisticsitems.model.dto.*;

import java.util.List;

/**
 * 统计条目(StatisticItem) dto、vo、do转换工具类
 *
 * @author lixing
 * @version V1.0 2021-07-27 16:11:59
 */
@Mapper(nullValueMappingStrategy = NullValueMappingStrategy.RETURN_DEFAULT,
        nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS)
public interface ConvertStatisticItemTool {

    ConvertStatisticItemTool INSTANCE = Mappers.getMapper(ConvertStatisticItemTool.class);

    /**
     * do转换为ResponseItemVO
     *
     * @param statisticItem do对象
     * @return ResponseItemVO
     * @author lixing
     * @version V1.0 2021-07-27 16:11:59
     */
    StatisticItemItemVO convert2ResponseItemDTO(StatisticItem statisticItem);

    /**
     * do转换为ResponseListItemVO
     *
     * @param statisticItem do对象
     * @return ResponseListItemVO
     * @author lixing
     * @version V1.0 2021-07-27 16:11:59
     */
    StatisticItemListItemVO convert2ResponseListItemDTO(StatisticItem statisticItem);

    /**
     * do列表转换为ResponseListItemVO列表
     *
     * @param statisticItemList do列表
     * @return ResponseListItemVO列表
     * @author lixing
     * @version V1.0 2021-07-27 16:11:59
     */
    List<StatisticItemListItemVO> convert2List(List<StatisticItem> statisticItemList);

    /**
     * addDTO转换为do
     *
     * @param addStatisticItemDTO addDTO
     * @return 要创建的do对象
     * @author lixing
     * @version V1.0 2021-07-27 16:11:59
     */
    StatisticItem convertAddDto2Entity(AddStatisticItemDTO addStatisticItemDTO);

    /**
     * updateDTO转换为实体
     *
     * @param statisticItem          更新前的do对象
     * @param updateStatisticItemDTO updateDTO
     * @return 更新后的do对象
     * @author lixing
     * @version V1.0 2021-07-27 16:11:59
     */
    StatisticItem convertUpdateDto2Entity(@MappingTarget StatisticItem statisticItem, UpdateStatisticItemDTO updateStatisticItemDTO);
}

